/*
 *    servo.c
 *
 *    Copyright (C) 2008-2011 Yvan Roch <yvan.roch@gmail.com>
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    This driver manages the FPGA R/C servos controller for the APF9328 and APF27
 *    Armadeus boards.
 */

/*
 * RELEASE-0.0.0 : 	Initial version for 2.6.23.1 Linux Kernel. This version works well
 * 					with Linux Kernel < 2.6.26
 *
 * RELEASE-0.0.1 : 	Correction for Linux Kernel >= 2.6.26. The class_device was
 * 					removed from the kernel tree since 2.6.26. This structure is an encapsulation
 * 					of a device that appears in /sys/class/XXX/XXX.Y filesystem
 * 					Before Kernel 2.6.26, this is the right way to create directories
 * 					and files in /sys/class/ filesystem and
 * 					communicate with the user space. class_device had a member parent
 * 					and it was possible to create a hierarchy of class_device.
 * 					Since Linux Kernel 2.6.26, this structure was gone and the right
 * 					way to create directory and file in /sys/class/ filesystem is the
 * 					more generic device structure.
 * 					A new design is implemented. There is 2 classes:
 * 						- servo_controller_class
 * 						- servo_class
 *					/sys/class/servo_controller_class/servo_controller_class.0 is the
 *					class entry for the servo controller.
 *					/sys/class/servo/servo.[0-23] are the entries for each R/C servo
 *					plugged to the servo controller.
 *
 * RELEASE-0.0.2 :	- 2011-03-08
 *					- Support both APF9328 and APF27
 *					- use readw() and writew() instead ioread16() and iowrite16()
 *					- use ARMADEUS_FPGA_BASE_ADDR_VIRT and ARMADEUS_FPGA_BASE_ADDR_PHYS
 */
#include "servo.h"

/*
 * This function is called when a servo_controller class device is released
 */
static void servo_controller_release(struct device *dev) {
    return;
}

/*
 * Declaration of the class servo_contoller used for the user interface
 * This class will appear under /sys/class/servo
 */
static struct class servo_controller_class = {
    .name       = SERVO_CONTROLLER_DRIVER_NAME,
    .dev_release = servo_controller_release,
    .owner = THIS_MODULE,
};

/*
 * This function is called when a servo class device is released
 */
static void servo_release(struct device *dev) {
    return;
}

/*
 * Declaration of the class servo used for the user interface
 * This class will appear under /sys/class/servo
 */
static struct class servo_class = {
    .name       = SERVO_DRIVER_NAME,
    .dev_release = servo_release,
    .owner = THIS_MODULE,
};

/*
 * Send a hardware reset command to the FPGA firmware
 */
static void reset(void){
    int i;
	/* Protects update against Race Condition */
	spin_lock(&spd->servo_spinlock);
	/* reset generation */
    writew(1,SERVO_COMMAND_ADDR);
    /* wait 10 ms */
    mdelay(10);
    writew(0,SERVO_COMMAND_ADDR);
    /* Reset the internal data */
    for (i=0;i<spd->nbr_servos;i++){
    	sepd[i]->desired_position=SERVO_MEDIUM_POSITION;
    	sepd[i]->upper_boundary=SERVO_UPPER_BOUNDARY;
    	sepd[i]->lower_boundary=SERVO_LOWER_BOUNDARY;
    	sepd[i]->offset=SERVO_OFFSET;
    	sepd[i]->speed_step=0;
    }
    spd->desired_servos_enable=0;
    /* Release the lock */
    spin_unlock(&spd->servo_spinlock);
}

/*
 * This function is triggered when the timer
 * spd->servo_timer is over. This function performs all the
 * hardware accesses except for hardware probe, initialization and reset.
 */
void servo_timer_function(unsigned long arg){
	int i;
	/* Look at the shutdown flag: the module is unloading */
	if (unlikely(spd->shutting_down_flag==1)){
		/* Do nothing and return without restarting the timer */
		return;
	}
	/* Look at the desired_servos_enable and put servos to the desired state */
	/* In most case spd->desired_servos_enable == spd->current_servos_enable */
	if (unlikely(spd->desired_servos_enable != spd->current_servos_enable)){
		/* Protect update against Race Condition */
		spin_lock(&spd->servo_spinlock);
		PDEBUG("spd->desired_servos_enable %u spd->current_servos_enable %u\n",spd->desired_servos_enable,spd->current_servos_enable);
		writew((u16)spd->desired_servos_enable,SERVO_ENABLE_LSB_ADDR);
		writew((u16)spd->desired_servos_enable >> 16,SERVO_ENABLE_MSB_ADDR);
		spd->current_servos_enable=spd->desired_servos_enable;
	    /* Release the lock */
	    spin_unlock(&spd->servo_spinlock);
	}
	/* Look at the real desired_position and put servos to the desired position */
	for (i=0;i<spd->nbr_servos;i++){
    	/* In most case sepd[i]->real_desired_position == sepd[i]->current_position */
		if (unlikely(sepd[i]->real_desired_position != sepd[i]->current_position)){
			/* Update needed: Protects update against Race Condition */
			spin_lock(&spd->servo_spinlock);
			/* First case: If speed control is off */
			if (likely(sepd[i]->speed_step==0)){
	    		sepd[i]->current_position=sepd[i]->real_desired_position;
			}
			/* Second case where speed control is used */
			else {
				/* Clockwise case */
				if (sepd[i]->current_position < sepd[i]->real_desired_position){
					/* The delta is bigger than speed_step */
					if((sepd[i]->real_desired_position-sepd[i]->current_position)>=sepd[i]->speed_step){
						sepd[i]->current_position += sepd[i]->speed_step;
					}else {
						sepd[i]->current_position=sepd[i]->real_desired_position;
					}
				}
				/* Counterclockwise */
				else{
					/* The delta is bigger than speed_step */
					if((sepd[i]->current_position-sepd[i]->real_desired_position)>=sepd[i]->speed_step){
						sepd[i]->current_position -= sepd[i]->speed_step;
					}else {
						sepd[i]->current_position=sepd[i]->real_desired_position;
					}
				}
			}
    	    /* I/O Write of new current_position */
			writew(sepd[i]->current_position,SERVO_POSITION_BASE+(sepd[i]->id*2));
			/* Release the lock */
    	    spin_unlock(&spd->servo_spinlock);
    	}
    }
	/* Restart the time */
	spd->servo_timer.expires += SERVO_WAIT_TICKS;
	add_timer(&spd->servo_timer);
}
static void enable_servo(u8 id){
	u32 mask32=0x00000001;
	if (id >= spd->nbr_servos ){
		/* this servo does not exist !!!*/
		PWARNING("try to enable an inexistent servo!!!\n");
		return;
	}
	/* set the spd->servos_enable bit for this servo*/
	mask32 = mask32 << id;
	/* Protects update against Race Condition */
	spin_lock(&spd->servo_spinlock);
	spd->desired_servos_enable=spd->desired_servos_enable | mask32;
	/* Release the lock */
	spin_unlock(&spd->servo_spinlock);
}
static void disable_servo(u8 id){
	u32 mask32=0x00000001;
	if (id >= spd->nbr_servos ){
		/* this servo does not exist !!!*/
		PWARNING("try to disable an inexistent servo!!!\n");
		return;
	}
	/* set the spd->servos_enable bit for this servo*/
	mask32 = mask32 << id;
	mask32 = ~mask32;
	/* Protects update against Race Condition */
	spin_lock(&spd->servo_spinlock);
	spd->desired_servos_enable=spd->desired_servos_enable & mask32;
	/* Release the lock */
	spin_unlock(&spd->servo_spinlock);
}

/**
 * servo_show_position - return the position of the servo managed by dev
 * @dev: device to work with
 * @buf: space to write the current value in
 */
static ssize_t servo_show_position(struct device *dev, struct device_attribute *attr, char *buf){
    ssize_t ret_size = 0;
    /* Get the servo_device that owns the device */
    struct servo_device *sepd=to_servo_device(dev);

    ret_size=sprintf(buf,"%u",sepd->desired_position);
    return ret_size;
}
/**
 * servo_store_position - store the position and put the servo to that position
 * @dev: device to work with
 * @buf: buffer with new value in ASCII
 * @size: size of buffer
 *
 * Returns the given @size
 */
static ssize_t servo_store_position(struct device *dev, struct device_attribute *attr, const char *buf, size_t size)
{
    int read_position;
	/* Get the servo_device that owns the device */
    struct servo_device *sepd=to_servo_device(dev);
	/* Protect update against Race Condition */
	spin_lock(&spd->servo_spinlock);
    read_position = simple_strtol(buf, NULL, 10);
    /* Check the boundaries constraints */
    if ((read_position+sepd->offset) < sepd->lower_boundary){
    	PINFO("Servo %u: Position entered %d with combination with Offset %d is lower than Lower boundary %u, adjust to Lower boundary.\n",sepd->id,read_position,sepd->offset,sepd->lower_boundary);
    	sepd->real_desired_position = sepd->lower_boundary;
    	sepd->desired_position = sepd->real_desired_position-sepd->offset;
    } else if ((read_position+sepd->offset) > sepd->upper_boundary){
    	PINFO("Servo %u: Position entered %d with combination with Offset %d is greater than Upper boundary %u, adjust to Upper boundary.\n",sepd->id,read_position,sepd->offset,sepd->upper_boundary);
    	sepd->real_desired_position = sepd->upper_boundary;
    	sepd->desired_position = sepd->real_desired_position-sepd->offset;
    }
    /* The right case */
    else {
    	sepd->desired_position=read_position;
    	sepd->real_desired_position = sepd->desired_position+sepd->offset;
    }
    /* Release the lock */
	spin_unlock(&spd->servo_spinlock);
   	PDEBUG("Put servo %u to %d position\n",sepd->id,sepd->desired_position);
    return size;
}
/**
 * servo_show_enable - return the enable of the servo managed by dev
 * @dev: device to work with
 * @buf: space to write the current value in
 */
static ssize_t servo_show_enable(struct device *dev, struct device_attribute *attr, char *buf){
    ssize_t ret_size = 0;
    u32 mask32=0x00000001;
    u32 value;
    /* Get the servo_device that owns the device */
    struct servo_device *sepd=to_servo_device(dev);
    mask32 = mask32 << sepd->id;
    value=spd->desired_servos_enable & mask32;
    value=value >> sepd->id;
    ret_size=sprintf(buf,"%u",value);
    return ret_size;
}
/**
 * servo_store_enable - store the enable and put the servo to that enable
 * @dev: device to work with
 * @buf: buffer with new value in ASCII
 * @size: size of buffer
 *
 * Returns the given @size
 */
static ssize_t servo_store_enable(struct device *dev, struct device_attribute *attr, const char *buf, size_t size)
{
    int value;
	/* Get the servo_device that owns the device */
    struct servo_device *sepd=to_servo_device(dev);
    value=simple_strtol(buf, NULL, 10);
    if (value==1){
    	PDEBUG("Enable servo %u to %u\n",sepd->id,sepd->real_desired_position);
    	enable_servo(sepd->id);
    }
    if (value==0){
    	PDEBUG("Disable servo %u\n",sepd->id);
    	disable_servo(sepd->id);
    }
    return size;
}
/**
 * servo_show_upper_boundary - return the upper_boundary of the servo managed by dev
 * @dev: device to work with
 * @buf: space to write the current value in
 */
static ssize_t servo_show_upper_boundary(struct device *dev, struct device_attribute *attr, char *buf){
    ssize_t ret_size = 0;
    /* Get the servo_device that owns the device */
    struct servo_device *sepd=to_servo_device(dev);
    ret_size=sprintf(buf,"%u",sepd->upper_boundary);
    return ret_size;
}
/**
 * servo_store_upper_boundary - store the upper_boundary and put the servo to that upper_boundary
 * @dev: device to work with
 * @buf: buffer with new value in ASCII
 * @size: size of buffer
 *
 * Returns the given @size
 */
static ssize_t servo_store_upper_boundary(struct device *dev, struct device_attribute *attr, const char *buf, size_t size) {
    int read_upper_boundary;
	/* Get the servo_device that owns the device */
    struct servo_device *sepd=to_servo_device(dev);
	/* Protects update against Race Condition */
	spin_lock(&spd->servo_spinlock);
	read_upper_boundary = simple_strtol(buf, NULL, 10);
    /* Check the boundaries constraints */
	if (read_upper_boundary > SERVO_UPPER_BOUNDARY){
    	PINFO("Servo %u: Upper boundary entered %d is greater than SERVO_UPPER_BOUNDARY, adjust to SERVO_UPPER_BOUNDARY.\n",sepd->id,read_upper_boundary);
    	sepd->upper_boundary=SERVO_UPPER_BOUNDARY;
    }
	/* Check upper_boundary coherence against lower_boundary */
	else if (read_upper_boundary < sepd->lower_boundary){
		PINFO("Servo %u: Upper boundary entered %d is lower than Lower boundary %u, adjust to Lower boundary.\n",sepd->id,read_upper_boundary,sepd->lower_boundary);
    	sepd->upper_boundary=sepd->lower_boundary;
    }
	/* The right case */
	else {
    	sepd->upper_boundary=read_upper_boundary;
    }
    /* Check if real servo position satisfies the new constraint value */
    if (sepd->real_desired_position > sepd->upper_boundary){
    	PINFO("Servo %u: Real Desired Position %u is greater than Upper boundary %u, adjust to Upper boundary.\n",sepd->id,sepd->real_desired_position,sepd->upper_boundary);
    	sepd->real_desired_position = sepd->upper_boundary;
    	sepd->desired_position = sepd->real_desired_position-sepd->offset;
    }
    /* Release the lock */
	spin_unlock(&spd->servo_spinlock);
   	PDEBUG("Put servo %u upper boundary to %u.\n",sepd->id,sepd->upper_boundary);
    return size;
}

/**
 * servo_show_lower_boundary - return the lower_boundary of the servo managed by dev
 * @dev: device to work with
 * @buf: space to write the current value in
 */
static ssize_t servo_show_lower_boundary(struct device *dev, struct device_attribute *attr, char *buf){
    ssize_t ret_size = 0;
    /* Get the servo_device that owns the device */
    struct servo_device *sepd=to_servo_device(dev);
    ret_size=sprintf(buf,"%u",sepd->lower_boundary);
    return ret_size;
}
/**
 * servo_store_lower_boundary - store the lower_boundary and put the servo to that lower_boundary
 * @dev: device to work with
 * @buf: buffer with new value in ASCII
 * @size: size of buffer
 *
 * Returns the given @size
 */
static ssize_t servo_store_lower_boundary(struct device *dev, struct device_attribute *attr, const char *buf, size_t size)
{
    int read_lower_boundary;
	/* Get the servo_device that owns the device */
    struct servo_device *sepd=to_servo_device(dev);
	/* Protects update against Race Condition */
	spin_lock(&spd->servo_spinlock);
	read_lower_boundary = simple_strtol(buf, NULL, 10);
    /* Check the boundaries constraints */
    if (read_lower_boundary < SERVO_LOWER_BOUNDARY){
    	PINFO("Servo %u: Lower boundary entered %d is lower than SERVO_LOWER_BOUNDARY, adjust to SERVO_LOWER_BOUNDARY.\n",sepd->id,read_lower_boundary);
    	sepd->lower_boundary=SERVO_LOWER_BOUNDARY;
    }
	/* Check lower_boundary coherence against upper_boundary */
    else if (read_lower_boundary >sepd->upper_boundary){
    	PINFO("Servo %u: Lower boundary entered %d is greater than Upper boundary %u, adjust to Upper boundary.\n",sepd->id,read_lower_boundary,sepd->upper_boundary);
    	sepd->lower_boundary=sepd->upper_boundary;
    }
    /* The right case */
    else {
    	sepd->lower_boundary=read_lower_boundary;
    }
    /* Check if real servo position satisfies the new constraint value */
    if (sepd->real_desired_position < sepd->lower_boundary){
    	PINFO("Servo %u: Real Desired Position %u is lower than Lower boundary %u, adjust to Lower boundary.\n",sepd->id,sepd->real_desired_position,sepd->lower_boundary);
    	sepd->real_desired_position = sepd->lower_boundary;
    	sepd->desired_position = sepd->real_desired_position-sepd->offset;
    }
    /* Release the lock */
	spin_unlock(&spd->servo_spinlock);
   	PDEBUG("Put servo %u lower boundary to %u.\n",sepd->id,sepd->lower_boundary);
    return size;
}

/**
 * servo_show_offset - return the offset of the servo managed by dev
 * @dev: device to work with
 * @buf: space to write the current value in
 */
static ssize_t servo_show_offset(struct device *dev, struct device_attribute *attr, char *buf){
    ssize_t ret_size = 0;
    /* Get the servo_device that owns the device */
    struct servo_device *sepd=to_servo_device(dev);
    ret_size=sprintf(buf,"%d",sepd->offset);
    return ret_size;
}

/**
 * servo_store_offset - store the offset and put the servo to that offset
 * @dev: device to work with
 * @buf: buffer with new value in ASCII
 * @size: size of buffer
 *
 * Returns the given @size
 */
static ssize_t servo_store_offset(struct device *dev, struct device_attribute *attr, const char *buf, size_t size)
{
    int read_offset;
	/* Get the servo_device that owns the device */
    struct servo_device *sepd=to_servo_device(dev);
	/* Protects update against Race Condition */
	spin_lock(&spd->servo_spinlock);
	read_offset = simple_strtol(buf, NULL, 10);
    /* Check the bondaries constraints */
    if ((read_offset+sepd->desired_position) < sepd->lower_boundary){
    	PINFO("Servo %u: Offset entered %d with combination with Desired position %d is lower than Lower boundary %u, no Offset change.\n",\
    			sepd->id,read_offset,sepd->desired_position,sepd->lower_boundary);
    } else if ((read_offset+sepd->desired_position) > sepd->upper_boundary){
    	PINFO("Servo %u: Offset entered %d with combination with Desired position %d is greater than Upper boundary %u, no Offset change.\n",\
    			sepd->id,read_offset,sepd->desired_position,sepd->upper_boundary);
    }
    /* The right case */
    else {
    	sepd->offset=read_offset;
    	sepd->real_desired_position=sepd->desired_position+sepd->offset;
    }
	/* Release the lock */
	spin_unlock(&spd->servo_spinlock);
   	PDEBUG("Put servo %u Offset to %d.\n",sepd->id,sepd->offset);
    return size;
}

/**
 * servo_show_speed_step - return the speed_step of the servo managed by dev
 * @dev: device to work with
 * @buf: space to write the current value in
 */
static ssize_t servo_show_speed_step(struct device *dev, struct device_attribute *attr, char *buf){
    ssize_t ret_size = 0;
    /* Get the servo_device that owns the device */
    struct servo_device *sepd=to_servo_device(dev);
    ret_size=sprintf(buf,"%d",sepd->speed_step);
    return ret_size;
}

/**
 * servo_store_speed_step - store the speed_step and put the servo to that speed_step
 * @dev: device to work with
 * @buf: buffer with new value in ASCII
 * @size: size of buffer
 *
 * Returns the given @size
 */
static ssize_t servo_store_speed_step(struct device *dev, struct device_attribute *attr, const char *buf, size_t size)
{
    int read_speed_step;
	/* Get the servo_device that owns the device */
    struct servo_device *sepd=to_servo_device(dev);
	/* Protects update against Race Condition */
	spin_lock(&spd->servo_spinlock);
	read_speed_step = simple_strtol(buf, NULL, 10);
    /* Check the bondaries constraints */
    if ((read_speed_step < SERVO_SPEED_STEP_MIN) || (read_speed_step > SERVO_SPEED_STEP_MAX)){
    	PINFO("Servo %u: speed_step entered %d is out of bound\n",sepd->id,read_speed_step);
    }
    /* The right case */
    else {
    	sepd->speed_step=read_speed_step;
    }
	/* Release the lock */
	spin_unlock(&spd->servo_spinlock);
   	PDEBUG("Put servo %u speed_step to %d.\n",sepd->id,sepd->speed_step);
    return size;
}

/**
 * servo_show_current_position - return the current_position of the servo managed by dev
 * @dev: device to work with
 * @buf: space to write the current value in
 */
static ssize_t servo_show_current_position(struct device *dev, struct device_attribute *attr, char *buf){
    ssize_t ret_size = 0;
    /* Get the servo_device that owns the device */
    struct servo_device *sepd=to_servo_device(dev);
    ret_size=sprintf(buf,"%d",sepd->current_position);
    return ret_size;
}

static int __init servo_driver_probe(struct platform_device *pdev){
	int result;
	/* Memory allocation for the platform device */
	if (unlikely((sepd[pdev->id] = kmalloc(sizeof(struct servo_device), GFP_KERNEL)) == NULL)) {
		PERR("couldn't allocate memory for servo_device %d\n",pdev->id);
		result = -ENOMEM;
		goto error_malloc_servo_device;
	}
	/* keep the id */
	PDEBUG("servo_driver_probe: servo %d\n",pdev->id);
	sepd[pdev->id]->id=pdev->id;
	/*
	 * Initialisation of the class_dev struct
	 */
	memset(&sepd[pdev->id]->class_dev, 0, sizeof(struct device));
	/*
	 * Register the device as servo_controller_class but specify this
	 * device as son of the master servo_contoller device
	 */
	sepd[pdev->id]->class_dev.class = &servo_class;
//	sepd[pdev->id]->class_dev.class = &servo_controller_class;
	/* This device will appear in /sys/class/servo-dev/servo.X
	 * The parent of this device is the device located in the platform device
	 * in argument of this callback function
	 */
	sepd[pdev->id]->class_dev.parent = &pdev->dev;
//	sepd[pdev->id]->class_dev.parent = &spd->class_dev;
    snprintf(sepd[pdev->id]->class_dev.bus_id, BUS_ID_SIZE, "%s%i",SERVO_DRIVER_NAME, pdev->id);
    result = device_register(&sepd[pdev->id]->class_dev);
    if (unlikely(result)) {
    	PERR("%s class registering failed for servo_device %d\n", SERVO_DRIVER_NAME,pdev->id);
    	goto error_class_device_register;
    }
	/* Memory allocation for the attributes attr_servo_position*/
	if (unlikely((sepd[pdev->id]->attr_servo_position = kmalloc(sizeof(struct device_attribute), GFP_KERNEL)) == NULL)) {
		PERR("couldn't allocate memory for servo_device %d attr_servo_position\n",pdev->id);
		result = -ENOMEM;
		goto error_malloc_class_device_attribute_position;
	}
	/* fill the attribute fields for position attribute */
	sepd[pdev->id]->attr_servo_position->attr.name = "desired_position";
	sepd[pdev->id]->attr_servo_position->attr.mode = 0644;
	sepd[pdev->id]->attr_servo_position->attr.owner = THIS_MODULE;
	sepd[pdev->id]->attr_servo_position->show = servo_show_position;
	sepd[pdev->id]->attr_servo_position->store = servo_store_position;
	/* Register the attributes */
	result = device_create_file( &sepd[pdev->id]->class_dev, sepd[pdev->id]->attr_servo_position );
	if (unlikely(result)) {
		PERR("%s %d class_device_create_file attr_servo_position failed\n", SERVO_DRIVER_NAME,pdev->id);
    	goto error_class_device_create_file_position;
    }
	/* Memory allocation for the attributes attr_servo_enable*/
	if (unlikely((sepd[pdev->id]->attr_servo_enable = kmalloc(sizeof(struct device_attribute), GFP_KERNEL)) == NULL)) {
		PERR("couldn't allocate memory for servo_device %d attr_servo_enable\n",pdev->id);
		result = -ENOMEM;
		goto error_malloc_class_device_attribute_enable;
	}
	/* fill the attribute fields for enable attribute */
	sepd[pdev->id]->attr_servo_enable->attr.name = "enable";
	sepd[pdev->id]->attr_servo_enable->attr.mode = 0644;
	sepd[pdev->id]->attr_servo_enable->attr.owner = THIS_MODULE;
	sepd[pdev->id]->attr_servo_enable->show = servo_show_enable;
	sepd[pdev->id]->attr_servo_enable->store = servo_store_enable;
	/* Register the attributes */
	result = device_create_file( &sepd[pdev->id]->class_dev, sepd[pdev->id]->attr_servo_enable );
	if (unlikely(result)) {
		PERR("%s %d class_device_create_file attr_servo_enable failed\n", SERVO_DRIVER_NAME,pdev->id);
    	goto error_class_device_create_file_enable;
    }
	/* Memory allocation for the attributes attr_servo_upper_boundary*/
	if (unlikely((sepd[pdev->id]->attr_servo_upper_boundary = kmalloc(sizeof(struct device_attribute), GFP_KERNEL)) == NULL)) {
		PERR("couldn't allocate memory for servo_device %d attr_servo_upper_boundary\n",pdev->id);
		result = -ENOMEM;
		goto error_malloc_class_device_attribute_upper_boundary;
	}
	/* fill the attribute fields for upper_boundary attribute */
	sepd[pdev->id]->attr_servo_upper_boundary->attr.name = "upper_boundary";
	sepd[pdev->id]->attr_servo_upper_boundary->attr.mode = 0644;
	sepd[pdev->id]->attr_servo_upper_boundary->attr.owner = THIS_MODULE;
	sepd[pdev->id]->attr_servo_upper_boundary->show = servo_show_upper_boundary;
	sepd[pdev->id]->attr_servo_upper_boundary->store = servo_store_upper_boundary;
	/* Register the attributes */
	result = device_create_file( &sepd[pdev->id]->class_dev, sepd[pdev->id]->attr_servo_upper_boundary );
	if (unlikely(result)) {
		PERR("%s %d class_device_create_file attr_servo_upper_boundary failed\n", SERVO_DRIVER_NAME,pdev->id);
    	goto error_class_device_create_file_upper_boundary;
    }
	/* Memory allocation for the attributes attr_servo_lower_boundary*/
	if (unlikely((sepd[pdev->id]->attr_servo_lower_boundary = kmalloc(sizeof(struct device_attribute), GFP_KERNEL)) == NULL)) {
		PERR("couldn't allocate memory for servo_device %d attr_servo_lower_boundary\n",pdev->id);
		result = -ENOMEM;
		goto error_malloc_class_device_attribute_lower_boundary;
	}
	/* fill the attribute fields for lower_boundary attribute */
	sepd[pdev->id]->attr_servo_lower_boundary->attr.name = "lower_boundary";
	sepd[pdev->id]->attr_servo_lower_boundary->attr.mode = 0644;
	sepd[pdev->id]->attr_servo_lower_boundary->attr.owner = THIS_MODULE;
	sepd[pdev->id]->attr_servo_lower_boundary->show = servo_show_lower_boundary;
	sepd[pdev->id]->attr_servo_lower_boundary->store = servo_store_lower_boundary;
	/* Register the attributes */
	result = device_create_file( &sepd[pdev->id]->class_dev, sepd[pdev->id]->attr_servo_lower_boundary );
	if (unlikely(result)) {
		PERR("%s %d class_device_create_file attr_servo_lower_boundary failed\n", SERVO_DRIVER_NAME,pdev->id);
    	goto error_class_device_create_file_lower_boundary;
    }
	/* Memory allocation for the attributes attr_servo_offset*/
	if (unlikely((sepd[pdev->id]->attr_servo_offset = kmalloc(sizeof(struct device_attribute), GFP_KERNEL)) == NULL)) {
		PERR("couldn't allocate memory for servo_device %d attr_servo_offset\n",pdev->id);
		result = -ENOMEM;
		goto error_malloc_class_device_attribute_offset;
	}
	/* fill the attribute fields for offset attribute */
	sepd[pdev->id]->attr_servo_offset->attr.name = "offset";
	sepd[pdev->id]->attr_servo_offset->attr.mode = 0644;
	sepd[pdev->id]->attr_servo_offset->attr.owner = THIS_MODULE;
	sepd[pdev->id]->attr_servo_offset->show = servo_show_offset;
	sepd[pdev->id]->attr_servo_offset->store = servo_store_offset;
	/* Register the attributes */
	result = device_create_file( &sepd[pdev->id]->class_dev, sepd[pdev->id]->attr_servo_offset );
	if (unlikely(result)) {
		PERR("%s %d class_device_create_file attr_servo_offset failed\n", SERVO_DRIVER_NAME,pdev->id);
    	goto error_class_device_create_file_offset;
    }
	/* Memory allocation for the attributes attr_servo_speed_step*/
	if (unlikely((sepd[pdev->id]->attr_servo_speed_step = kmalloc(sizeof(struct device_attribute), GFP_KERNEL)) == NULL)) {
		PERR("couldn't allocate memory for servo_device %d attr_servo_speed_step\n",pdev->id);
		result = -ENOMEM;
		goto error_malloc_class_device_attribute_speed_step;
	}
	/* fill the attribute fields for offset attribute */
	sepd[pdev->id]->attr_servo_speed_step->attr.name = "speed_step";
	sepd[pdev->id]->attr_servo_speed_step->attr.mode = 0644;
	sepd[pdev->id]->attr_servo_speed_step->attr.owner = THIS_MODULE;
	sepd[pdev->id]->attr_servo_speed_step->show = servo_show_speed_step;
	sepd[pdev->id]->attr_servo_speed_step->store = servo_store_speed_step;
	/* Register the attributes */
	result = device_create_file( &sepd[pdev->id]->class_dev, sepd[pdev->id]->attr_servo_speed_step );
	if (unlikely(result)) {
		PERR("%s %d class_device_create_file attr_servo_speed_step failed\n", SERVO_DRIVER_NAME,pdev->id);
    	goto error_class_device_create_file_speed_step;
    }
	/* Memory allocation for the attributes attr_servo_current_position*/
	if (unlikely((sepd[pdev->id]->attr_servo_current_position = kmalloc(sizeof(struct device_attribute), GFP_KERNEL)) == NULL)) {
		PERR("couldn't allocate memory for servo_device %d attr_servo_current_position\n",pdev->id);
		result = -ENOMEM;
		goto error_malloc_class_device_attribute_current_position;
	}
	/* fill the attribute fields for offset attribute */
	sepd[pdev->id]->attr_servo_current_position->attr.name = "current_position";
	sepd[pdev->id]->attr_servo_current_position->attr.mode = 0444;
	sepd[pdev->id]->attr_servo_current_position->attr.owner = THIS_MODULE;
	sepd[pdev->id]->attr_servo_current_position->show = servo_show_current_position;
	/* Register the attributes */
	result = device_create_file( &sepd[pdev->id]->class_dev, sepd[pdev->id]->attr_servo_current_position );
	if (unlikely(result)) {
		PERR("%s %d class_device_create_file attr_servo_current_position failed\n", SERVO_DRIVER_NAME,pdev->id);
    	goto error_class_device_create_file_current_position;
    }


	/* Init the driver private data */
    pdev->dev.driver_data=sepd[pdev->id];
    /* Init pdev member */
    sepd[pdev->id]->pdev=pdev;
    /* Init boudaries */
    sepd[pdev->id]->upper_boundary=SERVO_UPPER_BOUNDARY;
    sepd[pdev->id]->lower_boundary=SERVO_LOWER_BOUNDARY;
    /* Init position */
	sepd[pdev->id]->desired_position=SERVO_MEDIUM_POSITION; /* medium position */
	sepd[pdev->id]->current_position=0;
	/* Init offset */
	sepd[pdev->id]->offset=SERVO_OFFSET;
	sepd[pdev->id]->real_desired_position=sepd[pdev->id]->desired_position+sepd[pdev->id]->offset;
	/* Init speed_step*/
	sepd[pdev->id]->speed_step=0;

	goto success;
error_class_device_create_file_current_position:
	kfree(sepd[pdev->id]->attr_servo_current_position);
error_malloc_class_device_attribute_current_position:
	device_remove_file( &sepd[pdev->id]->class_dev, sepd[pdev->id]->attr_servo_speed_step );
error_class_device_create_file_speed_step:
	kfree(sepd[pdev->id]->attr_servo_speed_step);
error_malloc_class_device_attribute_speed_step:
	device_remove_file( &sepd[pdev->id]->class_dev, sepd[pdev->id]->attr_servo_offset );
error_class_device_create_file_offset:
	kfree(sepd[pdev->id]->attr_servo_offset);
error_malloc_class_device_attribute_offset:
	device_remove_file( &sepd[pdev->id]->class_dev, sepd[pdev->id]->attr_servo_lower_boundary );
error_class_device_create_file_lower_boundary:
	kfree(sepd[pdev->id]->attr_servo_lower_boundary);
error_malloc_class_device_attribute_lower_boundary:
	device_remove_file( &sepd[pdev->id]->class_dev, sepd[pdev->id]->attr_servo_upper_boundary );
error_class_device_create_file_upper_boundary:
	kfree(sepd[pdev->id]->attr_servo_upper_boundary);
error_malloc_class_device_attribute_upper_boundary:
	device_remove_file( &sepd[pdev->id]->class_dev, sepd[pdev->id]->attr_servo_enable );
error_class_device_create_file_enable:
	kfree(sepd[pdev->id]->attr_servo_enable);
error_malloc_class_device_attribute_enable:
	device_remove_file( &sepd[pdev->id]->class_dev, sepd[pdev->id]->attr_servo_position );
error_class_device_create_file_position:
	kfree(sepd[pdev->id]->attr_servo_position);
error_malloc_class_device_attribute_position:
	device_unregister(&sepd[pdev->id]->class_dev);
error_class_device_register:
	kfree(sepd[pdev->id]);
error_malloc_servo_device:
		return result;
success:
	    return 0;
}

static int servo_driver_remove(struct platform_device *pdev){
	/* Get the servo_contoller_device structure */
	struct servo_device *sepd=(struct servo_device*)pdev->dev.driver_data;

	/* Unregister sysfs attributes */
	device_remove_file( &sepd->class_dev, sepd->attr_servo_position );
	device_remove_file( &sepd->class_dev, sepd->attr_servo_enable );
	device_remove_file( &sepd->class_dev, sepd->attr_servo_upper_boundary );
	device_remove_file( &sepd->class_dev, sepd->attr_servo_lower_boundary );
	device_remove_file( &sepd->class_dev, sepd->attr_servo_offset );
	device_remove_file( &sepd->class_dev, sepd->attr_servo_speed_step );
	device_remove_file( &sepd->class_dev, sepd->attr_servo_current_position );

	/* Free memory for attr */
	kfree(sepd->attr_servo_position);
	kfree(sepd->attr_servo_enable);
	kfree(sepd->attr_servo_upper_boundary);
	kfree(sepd->attr_servo_lower_boundary);
	kfree(sepd->attr_servo_offset);
	kfree(sepd->attr_servo_speed_step);
	kfree(sepd->attr_servo_current_position);

	/* Unregister the class device */
	device_unregister(&sepd->class_dev);
	PDEBUG("servo_driver_remove()\n");
	return 0;
}
static int servo_driver_suspend (struct platform_device *pdev, pm_message_t state){
	return 0;
}
static int servo_driver_resume(struct platform_device *pdev){
	return 0;
}

static struct platform_driver servo_driver = {
	.remove     = servo_driver_remove,
	.suspend    = servo_driver_suspend,
	.resume     = servo_driver_resume,
	.driver     = {
			.name   = SERVO_DRIVER_NAME,
			.owner		= THIS_MODULE,
	},
};

/**
 * servo_contoller_show_firmware_version - return the firmware version
 * @dev: device to work with
 * @buf: space to write the current value in
 */
static ssize_t servo_contoller_show_firmware_version(struct device *dev, struct device_attribute *attr, char *buf)
{
    ssize_t ret_size = 0;
    /* Get the servo_contoller_device that owns the device */
    struct servo_contoller_device *spd=to_servo_contoller_device(dev);

    ret_size=sprintf(buf,"%0#2X",spd->firmware_version);
    return ret_size;
}

/**
 * servo_contoller_show_nbr_servos - return the number of servos managed by the controller
 * @dev: device to work with
 * @buf: space to write the current value in
 */
static ssize_t servo_contoller_show_nbr_servos(struct device *dev, struct device_attribute *attr, char *buf)
{
    ssize_t ret_size = 0;
    /* Get the servo_contoller_device that owns the device */
    struct servo_contoller_device *spd=to_servo_contoller_device(dev);

    ret_size=sprintf(buf,"%u",spd->nbr_servos);
    return ret_size;
}

/**
 * servo_contoller_store_reset - send a hardware reset to the FPGA device
 * @dev: device to work with
 * @buf: buffer with new value in ASCII
 * @size: size of buffer
 *
 * Returns the given @size
 */
static ssize_t servo_contoller_store_reset(struct device *dev, struct device_attribute *attr, const char *buf, size_t size)
{
    long value;
    value = simple_strtol(buf, NULL, 10);
    if (value == 1){
    	PINFO("Reset from sysfs\n");
    	/* reset generation */
    	reset();
    }
    return size;
}
/**
 * servo_contoller_show_enable - return spd->servos_enable
 * @dev: device to work with
 * @buf: space to write the current value in
 */
static ssize_t servo_contoller_show_enable(struct device *dev, struct device_attribute *attr, char *buf)
{
    int i;
    u32 value;
    /* Get the servo_contoller_device that owns the device */
    struct servo_contoller_device *spd=to_servo_contoller_device(dev);
    value=spd->desired_servos_enable;
    for (i=31;i>=0;i--){
    	if ((value & 0x00000001)==1){
    		buf[i]='1';
    	}else{
    		buf[i]='0';
    	}
    	value=value>>1;
    }
    return 32;
}

/**
 * servo_contoller_store_enable - store spd->servos_enable and sent it to hardware
 * @dev: device to work with
 * @buf: buffer with new value in ASCII
 * @size: size of buffer
 *
 * Returns the given @size
 */
static ssize_t servo_contoller_store_enable(struct device *dev, struct device_attribute *attr, const char *buf, size_t size)
{
    /* Get the servo_contoller_device that owns the device */
    struct servo_contoller_device *spd=to_servo_contoller_device(dev);
	/* Protects update against Race Condition */
	spin_lock(&spd->servo_spinlock);
    spd->desired_servos_enable = (u32)simple_strtol(buf, NULL, 2);
	/* Release the lock */
	spin_unlock(&spd->servo_spinlock);
	return size;
}
/*
 * Attribute: /sys/class/servo_contoller/servo_contollerX/firmware_version
 * Read only attribute
 */
static struct device_attribute attr_servo_contoller_firmware_version = {
    .attr  = { .name = "firmware_version", .mode = 0444, .owner = THIS_MODULE },
    .show  = servo_contoller_show_firmware_version,
};

/*
 * Attribute: /sys/class/servo_contoller/servo_contollerX/nbr_servos
 * Read only attribute
 */
static struct device_attribute attr_servo_contoller_nbr_servos = {
    .attr  = { .name = "nbr_servos", .mode = 0444, .owner = THIS_MODULE },
    .show  = servo_contoller_show_nbr_servos,
};

/*
 * Attribute: /sys/class/servo_contoller/servo_contollerX/reset
 * Write only attribute
 */
static struct device_attribute attr_servo_contoller_reset = {
    .attr  = { .name = "reset", .mode = 0200, .owner = THIS_MODULE },
    .store  = servo_contoller_store_reset,
};

/*
 * Attribute: /sys/class/servo_contoller/servo_contollerX/enable
 * nbr_servos bits of status enable/disable of each servo
 */
static struct device_attribute attr_servo_contoller_enable = {
    .attr = { .name = "enable", .mode = 0644, .owner = THIS_MODULE },
    .show = servo_contoller_show_enable,
    .store = servo_contoller_store_enable,
};
static inline void unregister_servo_controller_sys_file( struct servo_contoller_device *spd ){
    // Unregister /sys/class/servo_contoller/servo_contollerX/*  attributes
    device_remove_file( &spd->class_dev, &attr_servo_contoller_firmware_version );
    device_remove_file( &spd->class_dev, &attr_servo_contoller_nbr_servos );
    device_remove_file( &spd->class_dev, &attr_servo_contoller_reset );
    device_remove_file( &spd->class_dev, &attr_servo_contoller_enable   );
}
static int __init servo_controller_driver_probe(struct platform_device *pdev){
	int result=0;
	u16 magic_id;
	PDEBUG("Callback function servo_controller_driver_probe\n");
	/* Reading the Magic Id to know if the ad-hoc firmware is loaded in the FPGA*/
	magic_id = readw(SERVO_MAGIC_ID_ADDR);
	if (magic_id == SERVO_MAGIC_ID) {
		/*Magic ID OK*/
		PINFO("Magic ID OK %#X\n",magic_id);
	} else {
		/*Bad Magic ID: the proper firmware is not loaded in the FPGA*/
		PERR("Bad Magic ID %#X, exit...\n",magic_id);
		return -ENXIO;
	}
	/* Requested I/O memory */
	if (unlikely((mem = request_mem_region(ARMADEUS_FPGA_BASE_ADDR_PHYS,FPGA_CSx_SIZE,SERVO_CONTROLLER_DRIVER_NAME)) == NULL)) {
		PERR("couldn't register I/O memory\n");
		result = -ENOMEM;
		goto error_request_mem_region;
	}

	/* Memory allocation for the platform device */
	if (unlikely((spd = kmalloc(sizeof(struct servo_contoller_device), GFP_KERNEL)) == NULL)) {
		PERR("couldn't allocate memory for servo_contoller_device\n");
		result = -ENOMEM;
		goto error_malloc;
	}
	/*
	 * Initialization of the class_dev struct
	 */
	memset(&spd->class_dev, 0, sizeof(struct device));
	/* Spinlock Initialization */
	spin_lock_init(&spd->servo_spinlock);
	/* Timer Initialization */
	init_timer(&spd->servo_timer);

	/* Reading the R/C servo FPGA firmware version*/
	spd->firmware_version = readw(SERVO_FIRMWARE_VERSION_ADDR);
	PINFO("Firmware version %0#2X\n",spd->firmware_version);

	/* Reading the number of R/C servo managed by the firmware*/
	spd->nbr_servos = readw(SERVO_NBR_SERVOS_ADDR);
	PINFO("%d R/C servo(s) managed by the FPGA firmware\n",spd->nbr_servos);
	/*
	 * Register the device
	 */
	spd->class_dev.class = &servo_controller_class;
	spd->class_dev.parent = &pdev->dev;
    snprintf(spd->class_dev.bus_id, BUS_ID_SIZE, "%s%i",SERVO_CONTROLLER_DRIVER_NAME, pdev->id);
    result = device_register(&spd->class_dev);
    if (unlikely(result)) {
    	PERR("%s class registering failed\n", SERVO_CONTROLLER_DRIVER_NAME);
    	goto error_class_device_register;
    }

    /* Register the attributes */
    result |= device_create_file( &spd->class_dev, &attr_servo_contoller_firmware_version );
    if (unlikely(result)) {
    	PERR("%s class_device_create_file attr_servo_contoller_firmware_version failed\n", SERVO_CONTROLLER_DRIVER_NAME);
    	goto error_class_device_create_file;
    }
    result |= device_create_file( &spd->class_dev, &attr_servo_contoller_nbr_servos );
    if (unlikely(result)) {
    	PERR("%s class_device_create_file attr_servo_contoller_nbr_servos failed\n", SERVO_CONTROLLER_DRIVER_NAME);
    	goto error_class_device_create_file;
    }
    result |= device_create_file( &spd->class_dev, &attr_servo_contoller_reset );
    if (unlikely(result)) {
    	PERR("%s class_device_create_file attr_servo_contoller_reset failed\n", SERVO_CONTROLLER_DRIVER_NAME);
    	goto error_class_device_create_file;
    }
    result |= device_create_file( &spd->class_dev, &attr_servo_contoller_enable );
    if (unlikely(result)) {
    	PERR("%s class_device_create_file attr_servo_contoller_enable failed\n", SERVO_CONTROLLER_DRIVER_NAME);
    	goto error_class_device_create_file;
    }

    /* Init the driver private data */
    pdev->dev.driver_data=spd;
    /* Init pdev member */
    spd->pdev=pdev;
    /* Initialisation of member variables*/
    spd->desired_servos_enable=0;
    spd->current_servos_enable=0;
    goto success;

error_class_device_create_file:
	unregister_servo_controller_sys_file(spd);
error_class_device_register:
	kfree(spd);
error_malloc:
	/* Release I/O memories */
	release_mem_region(ARMADEUS_FPGA_BASE_ADDR_PHYS,FPGA_CSx_SIZE);
error_request_mem_region:
		return result;
success:
	    return 0;
}
static int servo_controller_driver_remove(struct platform_device *pdev){
	/* Get the servo_contoller_device structure */
	struct servo_contoller_device *spd=(struct servo_contoller_device*)pdev->dev.driver_data;
	/* Unregister sysfs attributes */
	unregister_servo_controller_sys_file(spd);
	/* Unregister the class device */
	device_unregister(&spd->class_dev);
	/* Release I/O memories */
	release_mem_region(ARMADEUS_FPGA_BASE_ADDR_PHYS,FPGA_CSx_SIZE);
	PDEBUG("servo_controller_driver_remove()\n");
	return 0;
}
static int servo_controller_driver_suspend (struct platform_device *pdev, pm_message_t state){
	return 0;
}
static int servo_controller_driver_resume(struct platform_device *pdev){
	return 0;
}

static struct platform_driver servo_controller_driver = {
	/* We don't declare probe in this structure because we register by platform_driver_probe()
	 * instead of platform_driver_register() for not hotpluggable device
	 */
	//	.probe      = servo_controller_driver_probe,
	.remove     = servo_controller_driver_remove,
	.suspend    = servo_controller_driver_suspend,
	.resume     = servo_controller_driver_resume,
	.driver     = {
			.name   = SERVO_CONTROLLER_DRIVER_NAME,
			.owner	= THIS_MODULE,
	},
};

static int __init servo_init(void) {
	int result;

	int i,j;
	/* Main servo controller platform_device */
	struct platform_device *servo_controller_platform_device;

	/*
	 * Temporary array of platform_device pointer to keep trace of already
	 * allocated servo platform_device for error handling because the servo
	 * platform_device are dynamically allocated on the base of the number
	 * of R/C servos managed by the FPGA firmware.
	 */
	struct platform_device **servo_platform_device;
	PINFO("Version %d.%d\n",SERVO_VERSION_MAJ,SERVO_VERSION_MIN);
	PDEBUG("HZ: %d\n",HZ);
	PDEBUG("SERVO_WAIT_TICKS: %d\n",SERVO_WAIT_TICKS);
	/* Register the servo_contoller class. This class appears in /sys/class/servo_contoller*/
	PDEBUG("Initializing servo_contoller class.\n");
    if (unlikely((result = class_register(&servo_controller_class)))) {
    	PERR("%s: couldn't register class, exiting\n", SERVO_CONTROLLER_DRIVER_NAME);
    	goto error_class_register;
    }
	/* Register the servo class. This class appears in /sys/class/servo*/
	PDEBUG("Initializing servo class.\n");
    if (unlikely((result = class_register(&servo_class)))) {
    	PERR("%s: couldn't register class, exiting\n", SERVO_DRIVER_NAME);
    	goto error_class_register;
    }
    /* Details about platform driver and device are in drivers/base/platform.c
     * and include/linux/platform_device.h in the kernel tree
     */

    /* Create the platform_device for the FPGA device servo_controller*/
    if (unlikely((servo_controller_platform_device=platform_device_alloc(SERVO_CONTROLLER_DRIVER_NAME, 0)) == NULL)) {
    	PERR("platform_device_alloc %s.0 failed.\n",SERVO_CONTROLLER_DRIVER_NAME);
    	result = -ENOMEM;
    	goto error_platform_device_alloc;
    }
    /* Add the created device to the platform subsystem*/
    /* Remark: With Kernel 2.6.23.1 platform_device_register works fine...
     * Since Kernel 2.6.27.2 platform_device_register causes an error:
     * kobject (XXXXXXXX): tried to init an initialized object, something is seriously wrong.
     * It seems that platform_device_register calls device_initialize() and
     * platform_device_add(). platform_device_alloc() calls already device_initialize()
     * It seems the reason of the "tried to init an initialized object".
     * Fixed by calling platform_device_add (second half of platform_device_register) only
     */
    if (unlikely((result = platform_device_add(servo_controller_platform_device)))) {
    	PERR("platform_device_register %s.0 failed.\n",SERVO_CONTROLLER_DRIVER_NAME);
    	goto error_platform_device_register;
    }
    /* Register the platform driver  Probe device
     * Use platform_driver_probe() instead of platform_driver_register() for
     * not hotpluggable device.
     * This call trigger the callback function servo_controller_driver_probe of
     * the platform_driver servo_controller_driver
     */
    if (unlikely((result = platform_driver_probe( &servo_controller_driver,servo_controller_driver_probe )))) {
    	PERR("platform_driver_probe %s failed.\n", SERVO_CONTROLLER_DRIVER_NAME);
    	goto error_platform_driver_probe;
    }

    /*
     * Memory allocation for nbr_servos pointers to the nbr_servos servo_device
     * for each servo
     */
    if (unlikely((sepd = kmalloc(spd->nbr_servos*sizeof(struct servo_device *), GFP_KERNEL)) == NULL)) {
    	PERR("couldn't allocate memory for sepd\n");
    	result = -ENOMEM;
    	goto error_malloc_sepd;
    }
    /*
     * Memory allocation for nbr_servos pointers to the nbr_servos platform_device
     * to keep trace of already allocated platform_device for error handling
     */
    if (unlikely((servo_platform_device = kmalloc(spd->nbr_servos*sizeof(struct platform_device *), GFP_KERNEL)) == NULL)) {
    	PERR("couldn't allocate memory for servo_platform_device\n");
    	result = -ENOMEM;
    	goto error_malloc_platform_device;
    }
    /*
     * The allocation and registration of platform device for the nbr_servos are
     * made here. If this task is made in servo_controller_driver_probe() function
     * insmod will never return when inserting this module
     */

   	for (i=0;i<spd->nbr_servos;i++){
   		/* Create the platform_device for the FPGA device servo*/
   	    if (unlikely((servo_platform_device[i]=platform_device_alloc(SERVO_DRIVER_NAME, i)) == NULL)) {
   	    	PERR("platform_device_alloc %s.%d failed.\n",SERVO_DRIVER_NAME,i);
   	    	result = -ENOMEM;
   	    	goto error_servo_platform_device;
   	    }
   	    /* Add the created device to the platform subsystem*/
   	    /* Remark: With Kernel 2.6.23.1 platform_device_register works fine...
   	     * Since Kernel 2.6.27.2 platform_device_register causes an error:
   	     * kobject (XXXXXXXX): tried to init an initialized object, something is seriously wrong.
   	     * It seems that platform_device_register calls device_initialize() and
   	     * platform_device_add(). platform_device_alloc() calls already device_initialize()
   	     * It seems the reason of the "tried to init an initialized object".
   	     * Fixed by calling platform_device_add (second half of platform_device_register) only
   	     */
   	    if (unlikely((result = platform_device_add(servo_platform_device[i])))) {
   	    	PERR("platform_device_register %s.%d failed.\n",SERVO_DRIVER_NAME,i);
   	    	i++;
   	    	goto error_servo_platform_device;
   	    }
   	}
    /* Register the platform driver
     * Use platform_driver_probe() instead of platform_driver_register() for
     * not hotpluggable device.
     * This call trigger the callback function servo_driver_probe of
     * the platform_driver servo_controller_driver
     */
   	if (unlikely((result = platform_driver_probe( &servo_driver,servo_driver_probe )))) {
   		PERR("platform_driver_probe %s failed.\n", SERVO_DRIVER_NAME);
   		goto error_servo_platform_driver_probe;
   	}
   	/* It's All right... */
   	/* At this time servo_platform_device is not useful -> free */
   	kfree(servo_platform_device);
   	/* Put all servos on free wheel by reset */
	reset();
	/* Initialization of the shutting_down_flag flag */
	spd->shutting_down_flag=0;
    /* Start the timer */
    spd->servo_timer.data=0;
    spd->servo_timer.function=servo_timer_function;
    spd->servo_timer.expires=jiffies+SERVO_WAIT_TICKS;
    add_timer(&spd->servo_timer);
    goto success;

error_servo_platform_driver_probe:
error_servo_platform_device:
	/* Free the already allocated servo devices */
	for (j=0;j<i;j++){
		PDEBUG("platform_device_unregister %s.%d\n",SERVO_DRIVER_NAME,j);
		platform_device_unregister(servo_platform_device[j]);
	}
	kfree(servo_platform_device);
error_malloc_platform_device:
	kfree(sepd);
error_malloc_sepd:
	platform_driver_unregister ( &servo_controller_driver );
error_platform_driver_probe:
error_platform_device_register:
	platform_device_unregister(servo_controller_platform_device);
error_platform_device_alloc:
	class_unregister( &servo_controller_class );
error_class_register:
	return result;
success:
    return 0;
}

static void servo_cleanup(void) {
	int i;
	PDEBUG("servo_cleanup\n");
	 /* Put all servos on free wheel by reset */
	reset();
	/* Set the flag shutting_down_flag to avoid dead lock with timer function */
	spd->shutting_down_flag=1;
	/* Wait more than one timer cycle if timer function servo_timer_function is running */
	mdelay(30);
	/* Stop the timer */
	del_timer_sync(&spd->servo_timer);
	/* Release servo_driver */
	PDEBUG("0002\n");
	platform_driver_unregister ( &servo_driver );
	PDEBUG("0004\n");
	/* Release nbr_servos servo_platform_device platform device and free memory
	 * platform_device_unregister must be here. If platform_device_unregister call
	 * is made in the remove function of the platform driver... rmmod will never return... */
	for (i=0;i<spd->nbr_servos;i++){
		platform_device_unregister(sepd[i]->pdev);
		/* Free memory used by the platform device */
		kfree(sepd[i]);
	}
	/* Free the memory for **sepd*/
	kfree(sepd);
	PDEBUG("0010\n");
	/* Release the servo_controller_driver platform driver and free the memory */
	platform_driver_unregister ( &servo_controller_driver );
	PDEBUG("0020\n");
	/* Unregister the servo_controller_platform_device and free the memory
	 * platform_device_unregister must be here. If platform_device_unregister call
	 * is made in the remove function of the platform driver... rmmod will never return... */
	platform_device_unregister(spd->pdev);
	PDEBUG("0030\n");
	/* Free memory used by the platform device */
	kfree(spd);

	/* Unregister the servo_controller_class class */
	class_unregister( &servo_controller_class );
	class_unregister( &servo_class );
	PINFO("released\n");
}

module_init(servo_init);
module_exit(servo_cleanup);
