/*
 *    servo.h
 *
 *    Copyright (C) 2008-2011 Yvan Roch <yvan.roch@gmail.com>
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *    This driver manages the FPGA R/C servos controller for the APF9328 and APF27
 *    Armadeus boards.
 */

/* for module/drivers */
#include <linux/init.h>
#include <linux/module.h>

/* linux device model */
#include <linux/kobject.h>
#include <linux/sysfs.h>
#include <linux/sysdev.h>
#include <linux/device.h>

/* for the platform driver */
#include <linux/platform_device.h>

/* for spinlock */
#include <linux/spinlock.h>

/* for kernel timer */
#include <linux/timer.h>

/* for mdelay */
#include <linux/delay.h>

/* readw() writew() */
#include <asm/io.h>

/* For kernel versionning */
#include <linux/version.h>

#include <mach/hardware.h>

/* For FPGA addresses and FPGA addressable memory size for APF27*/
#ifdef CONFIG_MACH_APF27
#include <mach/fpga.h>
#define FPGA_CSx_SIZE (CS5_SIZE)
#endif

/* For FPGA addressable memory size for APF9328*/
#ifdef CONFIG_MACH_APF9328
#define FPGA_CSx_SIZE (IMX_CS1_SIZE)
#endif


#ifndef SERVO_H_
#define SERVO_H_

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Yvan ROCH <yvan.roch@gmail.com>");
MODULE_DESCRIPTION("Armadeus FPGA R/C Servo device driver");

#define SERVO_DEBUG


#ifdef SERVO_DEBUG
#define PDEBUG(fmt,args...) printk(KERN_DEBUG "Armadeus FPGA R/C servo driver DEBUG: " fmt, ##args)
#else
#define PDEBUG(fmt,args...)
#endif




#define PERR(fmt,args...) printk(KERN_ERR "Armadeus FPGA R/C servo driver: " fmt, ##args)
#define PWARNING(fmt,args...) printk(KERN_WARNING "Armadeus FPGA R/C servo driver: " fmt, ##args)
#define PINFO(fmt,args...) printk(KERN_INFO "Armadeus FPGA R/C servo driver: " fmt, ##args)

/* Define driver version */
#define SERVO_VERSION_MAJ 0
#define SERVO_VERSION_MIN 2
/* Define the firmware magic id for firmware recognition from the driver */
#define SERVO_MAGIC_ID 0x7207
/* Define I/O addresses */
#define SERVO_BASE_ADDR ARMADEUS_FPGA_BASE_ADDR_VIRT
#define SERVO_MAGIC_ID_ADDR SERVO_BASE_ADDR
#define SERVO_FIRMWARE_VERSION_ADDR SERVO_BASE_ADDR+2
#define SERVO_NBR_SERVOS_ADDR SERVO_BASE_ADDR+4
#define SERVO_STATUS_ADDR SERVO_BASE_ADDR+6
#define SERVO_COMMAND_ADDR SERVO_BASE_ADDR+8
#define SERVO_ENABLE_LSB_ADDR SERVO_BASE_ADDR+10
#define SERVO_ENABLE_MSB_ADDR SERVO_BASE_ADDR+12
#define SERVO_POSITION_BASE SERVO_BASE_ADDR+16

/* Define the servo medium position */
#define SERVO_MEDIUM_POSITION 2048
/* Define servos boundaries positions */
#define SERVO_UPPER_BOUNDARY 4095
#define SERVO_LOWER_BOUNDARY 0
/* Define default servos offset*/
#define SERVO_OFFSET 0
/* Define Min and Max speed_step*/
#define SERVO_SPEED_STEP_MIN 0
#define SERVO_SPEED_STEP_MAX 4096

/* Define the number of ticks between 2 lauch of update position function
 * The divide by 100 is used to avoid the use of float number to obtain 0.02s=20ms
 */
#define SERVO_WAIT_TICKS (2*HZ)/100

#define SERVO_CONTROLLER_DRIVER_NAME "servo_contoller"
#define SERVO_DRIVER_NAME "servo"
/* Main ServoContoller device structure */
struct servo_contoller_device{
	u16 firmware_version;	/*firmwareVersion read at SERVO_FIRMWARE_VERSION_ADDR*/
	u8 nbr_servos;	/*number of managed servos read at SERVO_NBR_SERVOS_ADDR*/
	u32 desired_servos_enable;	/* bit n is the state enable/disable of servo n*/
	u32 current_servos_enable;	/* bit n is the state enable/disable of servo n*/
	/* Spinlock declaration. This spinlock is used to avoid race condition when
	 * updating attribute from user space or from kernel timer. A spinlock is used
	 * because a semaphore or a mutex can not be used. Semaphore and mutexes can
	 * sleep, and that strictly forbidden in kernel timer context
	 */
	spinlock_t servo_spinlock;
	/* Timer declaration. The function of this timer is called every 20ms
	 * or 2 ticks on arm9 platform with HZ=100. The fonction look at the desired
	 * position and enable state and perform the ad-hoc I/O operation if needed
	 */
	struct timer_list servo_timer;
	/*The shutting_down_flag flag is used to avoid a dead lock on servo_spinlock
	 * when calling del_timer_sync. If servo_spinlock is hold when calling del_timer_sync,
	 * and timer function hold servo_spinlock, it resultes a dead lock. So, shutting_down_flag
	 * is checked by the timer function before hold servo_spinlock. If shutting_down_flag is
	 * set, timer function simply returns and does nothing.
	 * On the advice of LINUX DEVICE DRIVERS THIRD EDITION Chapter 7 P 200
	 * Jonathan Corbet, Alessandro Rubini, and Greg Kroah-Hartman
	 */
	u8 shutting_down_flag;
	struct platform_device *pdev; /* Platform device for this device */
	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,27)
		struct class_device class_dev; /* Class device for this device */
	#else
		struct device class_dev; /* Device for this device */
	#endif
};
#define to_servo_contoller_device(d) container_of(d, struct servo_contoller_device, class_dev)
struct servo_device{
	u8 id; /* servo id 0 to nbr_servos-1 */
	/* Current position of this servo
	 * If speed <> 0, current_position if updated by steps by the timer function
	 * to reach desided_position
	 */
	u16 current_position;
	/* Desired position of this servo whithout offset
	 * It is a signed value to cover all the possible range between lower_boundary
	 * and upper_boundary. So, desired_position+offset may reach lower_boundary=0
	 * with a positive offset, desired_position must be negative...
	 */
	s16 desired_position;
	/* Real Desired position of this servo whit offset = desired_position + offset */
	u16 real_desired_position;
	/* Upper boundary for this servo
	 * The servo will never be over this boundary */
	u16 upper_boundary;
	/* Lower boundary for this servo
	 * The servo will never be under this boundary */
	u16 lower_boundary;
	/* Offset for this servo: Real Position = desired_position + offset
	 * It may bi nice to ajust mecanical imprecision */
	s16 offset;
	/* In this implementation desired_position has a 12 bits resolution
	 * When changing position with an update on desired_position, there is two
	 * possible ways:
	 * 	- if speed_step is set to zero, the driver put the selected servo to the
	 * the new desired_position at the next run of the kernel timer function
	 * servo_timer_function (approximatively 20 ms)
	 * 	- if speed_step is set to a positive value, the driver increments or
	 * decrements by speed_step the current position of the selected servo at each
	 * run (approximatively 20 ms) of the kernel timer function, until the
	 * desired_position is reached.
	 * For example, if a servo has the position 1000 and speed_step is set to 10.
	 * The new desired_position is set to 2000, then the servo will reach the new
	 * position ((2000-1000)/10)*20 ms, i.e. 2 seconds later.
	 * If  speed_step <> 0, more speed_step is great, more the speed is great.
	 * But the maximum speed is get whith speed_step=0 (no speed control).*/
	u16 speed_step;
	/* The memory for the attributes of this device are dynamically allocated
	 * So we must keep the pointers to these  attributes to free the memory
	 * at when the module is unloaded */
	struct device_attribute *attr_servo_position;
	struct device_attribute *attr_servo_enable;
	struct device_attribute *attr_servo_upper_boundary;
	struct device_attribute *attr_servo_lower_boundary;
	struct device_attribute *attr_servo_offset;
	struct device_attribute *attr_servo_speed_step;
	struct device_attribute *attr_servo_current_position;
	struct platform_device *pdev; /* Platform device for this device */
	#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,27)
		struct class_device class_dev; /* Class device for this device */
	#else
		struct device class_dev; /* Device for this device */
	#endif
};
#define to_servo_device(d) container_of(d, struct servo_device, class_dev)
/*
 * pointer to fpga I/O Memory IMX_CS1_VIRT
 */
struct resource *mem;
/*
 * Pointer to the main structure servo_contoller_device
 */
struct servo_contoller_device *spd;
/*
 * Pointer to the array of pointer to each servo_device
 */

struct servo_device **sepd;

#endif /*SERVO_H_*/
