--------------------------------------------------------------------------------
-- Copyright (C) 2008-2011 Yvan Roch

-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.


-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--------------------------------------------------------------------------------
-- Engineer: Yvan ROCH
-- 
-- Create Date: 2007-12-03
-- Design Name:
-- Module Name: SERVO_TOP - structural 
-- Project Name:
-- Target Devices:
-- Armadeus APF9328 with Xilinx Spartan3
-- Armadeus APF27 with Xilinx Spartan3a
-- Tool versions: Xilinx ISE 9.2 to ISE 12.4
-- Description: Servo Contoller
--
-- Dependencies: COUNTER.vhd, PWC.vhd, RESET.vhd, SERVO_APF27.ucf, SERVO_APF9328.ucf
--
-- Revision: 
-- Revision 0.01 - File Created
-- Revision 0.02 - 2008-05-11
-- Application of sonzerro and Fabrice Mousset recommendations:
-- 		- All comments are in english.
-- 		- Synchronous design, using of a single clock and removing the writeValue
-- 		signal.
-- 		- Using a reset signal.
-- 		- Using of rising_edge.
-- 		- Using generic parameters (number of servos...).
-- 		- Using generator for multiple entity.
-- 		- Using integer with range.
-- Removing unedeed signal readAccessPulse.
-- Adding MagicID register for firmware recognition from Linux driver.
-- Adding Firmware Version register for versatile configuration of Linux driver.
-- Adding Number of servos register for versatile configuration of Linux driver.
-- Adding Command/Status register for send command (like reset) to the firmware.
-- Revision 0.03 - 2008-07-09
-- 		- Use IEEE.NUMERIC_STD only. Avoid the use of SYNOPSYS Library
-- 		IEEE.STD_LOGIC_UNSIGNED and IEEE.STD_LOGIC_ARITH.
-- 		- Synchronous design, using of a single clock (removing 4MHz unused clock)
-- 		- Asynchronous Reset
-- Revision 0.04 - 2011-03-08 
--		- Support both APF9328 and APF27
-- Additional Comments:
-- This project is a R/C servos controller FPGA Firmware for the Xilinx Spartan3 embedded 
-- on the Armadeus board. It is inspired from the project 
-- "Atmel AVR 2004 Design Contest Entry Project Number A3722" of Circuit Cellar
-- from Eric Gagnon :
-- http://www.circuitcellar.com/avr2004/HA3722.html
-- 
-- This controler is able to drive from 1 to 32 R/C servos. 
-- This number is controled by the generic parameter 'nbrServos'
-- 
-- Addresses Mapping :
-- Spartan3 Base Address on Armadeus Board:	0xXX000000
-- Magic ID Register:						0xXX000000
-- Firmware Version Register:				0xXX000002
-- Number of R/C servos managed:			0xXX000004
-- Status Register:							0xXX000006
-- Command Register:						0xXX000008
-- Enable Servo Register LSB:				0xXX00000A
-- Enable Servo Register MSB:				0xXX00000C
-- Base Address of Servos Position Register	0xXX000010
-- XX is 12 for the APF9328 and D6 for the APF27
-- 
-- Each servo has a 16 bits position register with only 12 signifiant bits that containt the servo position.
-- Example:
-- 0000 written at address 0xXX000010 will put the servo 0 on the maximal counterclockwise position.
-- 0800 written at address 0xXX000010 will put the servo 0 on the median position.
-- 0FFF written at address 0xXX000010 will put the servo 0 on the maximal clockwise position.
--
-- The Enable Servo Register LSB and Enable Servo Register MSB are the servos enable control registers:
-- XXX1 written at address 0xXX00000A will enable the servo 0 (the servo will keep his planed position)
-- XXX0 written at address 0xXX00000A will disable the servo 0 (the servo is freewheeling)
-- XXX1 written at address 0xXX00000C will enable the servo 16 (the servo will keep his planed position)
-- XXX0 written at address 0xXX00000C will disable the servo 16 (the servo is freewheeling)
--
-- Status Register:
-- bit0: If set, a write error occured
--
-- Command Register:
-- bit0: If set, the firmware will reset
--------------------------------------------------------------------------------

library IEEE;
-- IEEE.STD_LOGIC_UNSIGNED.ALL and IEEE.STD_LOGIC_ARITH.ALL are not used because
-- they may have some incompability with some synthesizer (SYNOPSYS library)
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity SERVO_TOP is
	-- nbrServos is the number of servos managed by this module
	-- nbrServos is not declared as a constant because it is used is the
	-- port declaration section of this module (the synthetizer does not permit that)
	-- When changing nbrServos you must modify the SERVO_APFXX.ucf file for connecting
	-- pwcOut<XX> properly.
 	generic ( nbrServos : integer := 32);
	port (
		-- MC9328MXL or i.MX27pins
		-- MC9328MXL or i.MX27 16 bits Data Bus
		Data : inout std_logic_vector(15 downto 0);
		
		-- MC9328MXL or i.MX27 Address Bus A1-A12
		Addr : in std_logic_vector(11 downto 0);
		
		-- Write access 
		-- APF9328: CTRL9, Spartan3 pin 60, MC9328MXL: EB3 pin M5 Active Low
		-- This signal from the MC9328MXL is active low for write operations SOLELY by default
		-- According to MC9328MXL Data Sheet p4:
		-- LSB Byte Strobe Active low external enable byte signal that controls D [7:0].
		-- See Write timing  Data Sheet p28
		-- See default configuration Reference Manual p11-17: Chip Select Control Registers bit 11 EBC
		--
		-- APF27: EB0N, Spartan3a pin P9, i.MX27: EB0 pin W16 Active Low
		-- This signal from the i.MX27 is active low for write operations SOLELY by default
		-- According to i.MX27 Data Sheet p27:
		-- Active low external enable byte signal that controls D [15:8], shared with PCMCIA PC_REG.
		-- See Write timing  Data Sheet p95
		-- See default configuration Reference Manual p17-18: CSCR0L Chip Select Control Registers bit 11 EBC
		WR : in std_logic;
		
		-- Read access 
		-- APF9328: CTRL10, Spartan3 pin 63, MC9328MXL: OE pin R5, Active Low
		-- This signal from the MC9328MXL is active low for read operation SOLELY
		-- According to MC9328MXL Data Sheet p4:
		-- Memory Output Enable Active low output enables external data bus.
		-- See Read timing Data Sheet P26
		--
		-- APF27: OEN, Spartan3a pin R9, i.MX27: OE pin Y17, Active Low
		-- This signal from the i.MX27 is active low for read operation SOLELY
		-- According to i.MX27 Data Sheet p27:
		-- Memory Output Enable-Active low output enables external data bus, shared with PCMCIA PC_IOWR.
		-- See Read timing Data Sheet P95
		RD : in std_logic;
		
		-- Chip select
		-- APF9328: CTRL1, Spartan3 pin 59, MC9328MXL: CS1 M8, Active Low
		-- APF27: CS5N, Spartan3a pin P10, i.MX27: CS5 W15, Active Low
		CS : in std_logic; 
		
		-- Clock input
		-- APF9328: at 96 MHz from MC9328MXL on Spartan3 pin P55
		-- APF27: at 100 MHz from i.MX27 on Spartan3a pin N9
		clkIn : in  STD_LOGIC;
		
		-- nbrServos 1 bit registers declaration for PWC outputs
		pwcOut : out std_logic_vector(nbrServos-1 downto 0)
	);
end entity SERVO_TOP;

architecture structural of SERVO_TOP is
	-- Contants declaration
	-- Magic ID
	constant regMagicID : std_logic_vector(15 downto 0) := X"7207";
	-- Firmware Version
	constant regFirmwareVersion : std_logic_vector(15 downto 0) := X"0004";
	-- MagicID register Address: 0xXX000000
	constant regMagicIDAddr : std_logic_vector(11 downto 0) := "000000000000";
	-- Firmaware version register Address:0xXX000002
	constant regFirmwareVersionAddr : std_logic_vector(11 downto 0) := "000000000001";
	-- Number of servos register Address: 0xXX000004
	constant regNumberServosAddr : std_logic_vector(11 downto 0) := "000000000010";
	-- Status register Address: 0xXX000006
	constant regStatusAddr : std_logic_vector(11 downto 0) := "000000000011";
	-- Command register Address: 0xXX000008
	constant regCmdAddr : std_logic_vector(11 downto 0) := "000000000100";
	-- Servos Validation Register LSB Address: 0xXX00000A
	constant regServosValidationLSBAddr : std_logic_vector(11 downto 0) := "000000000101";
	-- Servos Validation Register MSB Address: 0xXX00000C
	constant regServosValidationMSBAddr : std_logic_vector(11 downto 0) := "000000000110";
	-- Servos Position Registers base address : 0xXX000010
	-- Servo N Position Register address is : N*2 + addrRegServosPositionDase
	constant addrRegServosPositionBaseAddr : std_logic_vector(11 downto 0) := "000000001000";

	-- Type reg12Array declaration that containts nbrServos signals of 12 bits
	-- This type is used for the sorvos positions registers
	type reg12Array is array(0 to nbrServos-1) of std_logic_vector(11 downto 0);
	
	-- Signals declaration
	-- nbrServos 16 bits position registers declaration
	signal servosReg : reg12Array;
	-- nbrServos 1 bit enable registers declaration
	signal regEnable : std_logic_vector(31 downto 0);
	-- Status register
	signal regStatus : std_logic;
	-- Command register
	signal regCmd : std_logic_vector(15 downto 0);
	-- Reset active on high level from the RESET_MODULE
	signal reset : std_logic;
	-- Reset request send to the RESET_MODULE
	signal requestReset : std_logic;
	-- Signal buffer for the main counter
	signal count : std_logic_vector(15 downto 0);

-- Architecture definition
begin
 	-- Reset generator Instantiation
 	resetGenerator : entity work.RESET_MODULE(behavioral)
 	-- Port mapping for Reset generator
	port map (
		clkIn => clkIn,
		resetIn => requestReset,
		resetOut => reset
	);
	-- Counter instantiation
	counterGenerator : entity work.COUNTER_MODULE(behavioral)
	-- Clock division ratio setting
	-- Set ratio to 24 for a APF9328
	-- Set ratio to 25 for a APF27
	generic map (
		ratio => 24
	)
	-- Port mapping for Counter
	port map (
		clkIn => clkIn,
		counterOut16Bits => count
	);
	-- PWC Modules Instantiation
	PWC : for N in 0 to nbrServos-1 generate
		PWC_MOD : entity work.PWC_MODULE(behavioral)
			port map (
				count      => count,
				clkIn => clkIn,
				servosReg  => servosReg(N),
				enableReg  => regEnable(N),
				pwcOut     => pwcOut(N)
			);
	end generate PWC;

	-- Write process in the firmware registers from the P bus
	registerWrite: process (clkIn,reset)
	begin  -- process registerWrite
		-- General Initialisation Process
		-- This initialisation process is here because if it is located in
		-- an other process the synthesis will failed because there are multiple
		-- asignments to the same signals
		-- Asynchronous Reset
		if reset = '1' then
			requestReset <= '0';
			regStatus <= '0';
			regCmd <= X"0000";
			-- All servos are disable
			regEnable <= X"00000000";
			-- Medium position for all servos
			for N in 0 to nbrServos-1 loop
				servosReg(N) <= X"800";
			end loop;
		elsif rising_edge(clkIn) then
			-- Write access
			if CS = '0' and WR = '0' then
				-- Addresses decoding, base address is 0xXX000000
				case Addr is
					-- MagicID, Firmaware version, regNumberServos, Status
					-- register not processed: read only registers!!!
					when regCmdAddr =>
						-- Save in the regCmd
						regCmd <= Data(15 downto 0);
						-- Here: code for command execution...
						-- Software Reset
						if regCmd = X"0001" then
							requestReset <= '1';
						end if;
						if regCmd = X"0000" then
							requestReset <= '0';
						end if;
					-- Servos Validation Register LSB
					when regServosValidationLSBAddr =>
						-- save in regEnable LSB
						regEnable(15 downto 0) <= Data(15 downto 0);
					-- Servos Validation Register MSB
					when regServosValidationMSBAddr =>
						-- save in regEnable MSB
						regEnable(31 downto 16) <= Data(15 downto 0);
					when others => null;
					-- Add here read error processing
				end case;				
				-- Save position value for all servos
				for N in 0 to nbrServos-1 loop
					-- unigned cast must be used with IEEE.NUMERIC_STD
					if (std_logic_vector(unsigned(addrRegServosPositionBaseAddr) + N)=Addr) then
						-- Save in Servo N Position Register
						servosReg(N) <= Data(11 downto 0);
					end if;
				end loop;
			end if;
		end if;
	end process registerWrite;

	-- Read process of firmware registers from the P bus
	registerRead : process (clkIn)
	begin -- process registerRead
		if rising_edge(clkIn) then
			-- Read access (RD=0 et CS=0)
			if CS = '0' and RD = '0' then
				-- Read the general register:
				-- regMagicID,regFirmwareVersion, 	regNumberServos,regStatus
				-- regCmd,regServosValidationLSBAddr,regServosValidationMSBAddr
				case Addr is
					-- Read MagicID
					when regMagicIDAddr =>
						-- Send on the P data bus the MagicID
						Data <= regMagicID;
					-- Read Firmaware version
					when regFirmwareVersionAddr =>
						-- Send on the P data bus the Firmaware version
						Data <= regFirmwareVersion;
					-- Read regNumberServos
					when regNumberServosAddr =>
						-- Send on the P data bus the number of managed servos
						Data <= std_logic_vector(to_unsigned(nbrServos,16));
					-- Read regStatus
					when regStatusAddr =>
						-- Send on the P data bus the Status
						Data <= X"000" & "000" & regStatus;
					-- Read regCmd
					when regCmdAddr =>
						-- Send on the P data bus the last cms code
						Data <= regCmd;
					-- Servos Validation Register LSB
					when regServosValidationLSBAddr =>
						-- Send on the P data bus regEnable LSB
						Data <= regEnable(15 downto 0);
					-- Servos Validation Register MSB
					when regServosValidationMSBAddr =>
						-- Send on the P data bus regEnable MSB
						Data <= regEnable(31 downto 16);
					when others		=>
						Data <= (others => '0');
				end case;
				-- Read position value for all servos
				for N in 0 to nbrServos-1 loop
					-- unigned cast must be used with IEEE.NUMERIC_STD
					if (std_logic_vector(unsigned(addrRegServosPositionBaseAddr) + N)=Addr) then
						-- Send on the P data bus Servo N Position Register
						Data <= "0000" & servosReg(N)(11 downto 0);
					end if;
				end loop;
			else
				-- hig impedance si acces autre qu'en lecture
				Data <= (others => 'Z');
			end if;
		end if;
	end process registerRead;
end architecture structural;
